<?php
header("Access-Control-Allow-Origin: *");
header('Cache-Control: no-cache, must-revalidate'); 
header("Content-Type: text/html; charset=UTF-8");
header("HTTP/1.1 200 OK");

date_default_timezone_set("America/Sao_Paulo");

define('BASE_URL', 'https://teseprime.bitrix24.com/rest/1/l9y03778v4r7vhzd/');

require "./conexao.php";

/**
 * Write data to log file. (by default disabled)
 * WARNING: this method is only created for demonstration, never store log file in public folder
 *
 * @param mixed $data
 * @param string $title
 * @return bool
 */
function writeToLog($data, $title = '')
{

    $log = "\n------------------------\n";
    $log .= date("Y.m.d G:i:s")."\n";
    $log .= (strlen($title) > 0 ? $title : 'DEBUG')."\n";
    $log .= print_r($data, 1);
    $log .= "\n------------------------\n";

    file_put_contents("./handlerUser.log", $log, FILE_APPEND);

    return true;
}

/* Mostrar de forma formatada os dados de um vetor */
function printArray($array){
    
    echo "<pre>";
    print_r($array);
    echo "</pre>";
    
}

function pauseUs($time = 800000){
    usleep($time);
}

function getJson($url, $array, $print = 0){
    
    require "./conexao.php";
    
    global $idProcessando;
    global $obsProcessando;
    
    
    $queryData = http_build_query($array);

    $curl = curl_init();
	curl_setopt_array($curl, array(
								CURLOPT_SSL_VERIFYPEER => 0,
								CURLOPT_POST => 1,
								CURLOPT_HEADER => 0,
								CURLOPT_RETURNTRANSFER => 1,							
								CURLOPT_URL => BASE_URL.$url,
								CURLOPT_POSTFIELDS => $queryData,
								));
	$result = curl_exec($curl);
	curl_close($curl);

	$result = json_decode($result, 1);
	
	if (array_key_exists('error', $result)){

        echo "Error: ".$result['error_description']."<br/>";
        
        updateStatus($idProcessando, $db, 3, $obsProcessando, "Error: ".$result['error_description']);

        writeToLog($result, "Error: ");
        
        exit();

    }
	
	if($print == 1){
	    printArray($url);
	    printArray($array);
	    printArray($result);

	}
	
	pauseUs();
	
	$db = null;

	return $result;
    
}




function getContact($cpfBusca, $print = 0){
    
    $queryUrl = "crm.contact.list";

    $queryArray = array("filter" => array(
                                    "=UF_CRM_1589202720035" => $cpfBusca,
                        ),
                        "order" => array("ID", "ASC"),
                        "select" => array('*', 'UF_*', 'PHONE', 'EMAIL')
                        );  

    $result = getJson($queryUrl, $queryArray, $print);

	return $result;
}



// Chamada da função
// "UF_CRM_61EEF0C9C15A3"  => getField('UF_CRM_61EEF0C9C15A3', $dados['estado'], 'contact', 0, MB_CASE_UPPER), o MB_CASE_UPPER (TUDO MAIÚSCULO) - MB_CASE_TITLE (Primeia Letra Maiúscula)
function getField($field, $item, $entity, $print = 0){
    
    // $item = trim(mb_convert_case($item, $case, 'UTF-8'));
    
    printArray($field);
    printArray($item);
    
    if(trim($item) != ''){
    
        $queryUrl = "crm.{$entity}.userfield.list";
    
        $array = array(
                "filter" => array(
                        "FIELD_NAME" => $field
                    )
            );
    
        $result = getJson($queryUrl, $array, $print);
        
        if(isset($result['result'][0]['LIST'][0]['ID'])){
            foreach($result['result'][0]['LIST'] as $key => $value){
                
                $itens[$key] = trim($value['VALUE']);
                $itensId[$key] = $value['ID'];
                        
            }
            $found = array_search(trim($item), $itens);
        }else{
            $found = false;
        }
        
        if($found === false){
    
            updateField($result['result'][0]['ID'], trim($item), $entity, $print);
            $newItem = getField($field,  trim($item), $entity, $print);
            
            return $newItem;
    
        }else{
    
            return $itensId[$found];
        }
    }else{
        // getField($field, 'Não Informado', $entity, $print);
        return "";
    }

}

function updateField($fieldId, $item, $entity, $print =0){
    
    $queryUrl = "crm.{$entity}.userfield.update";
    
    $array = array(
            "ID" => $fieldId,
            "fields" => array(
                        "LIST" => array(array(
                                     "VALUE" => $item
                                    ))
                        ),
        );

    $result = getJson($queryUrl, $array, $print);
    sleep(2);
    
}


function addFeed($company, $mensagem, $entity, $print = 1){
    
    $queryUrl = "crm.livefeedmessage.add";

    $queryArray = array(
								 'fields' => array(

								 	"POST_TITLE" => $company,
								 	"MESSAGE" => $mensagem,
								 	"ENTITYTYPEID" =>  $entity,
								 	"ENTITYID" => $company
								 
								 ),
				);  

    $result = getJson($queryUrl, $queryArray, $print);

	return $result;
    
}



function getRequestBd($db){
    
    $dados = array();

    $sql = $db->prepare("SELECT * FROM bitrixToDrupal where status = 0 order by id limit 20");
    $sql->execute();
    
    if($sql->rowCount() > 0){
        
        $dados = $sql->fetchAll();
        
        return $dados;
        
    }else{
        
        echo "Sem Atualizações";
        exit();
        
    }
}

function checkStatus($id, $db, $status){
    
    $query = "SELECT * FROM bitrixToDrupal where status = 0 and id = {$id}";
    
    printArray($query);
    
    $sql = $db->prepare($query);
    $sql->execute();
    
    if($sql->rowCount() > 0){
        
        return true;
        
    }else{
        
        return false;
        
    }
    
}

function updateStatus($id, $db, $status, $oldObs, $newObs = null){
    
    if(isset($newObs)){
        $obs = unserialize($oldObs);
        array_push($obs, $newObs);
        $obs = serialize($obs);
    }else{
        $obs = $oldObs;
    }

    $query = "update bitrixToDrupal set status = {$status}, updated=CURRENT_TIMESTAMP(), obs = '".str_replace("'", "-", $obs)."' where id = ".$id;

    printArray($query);
    
    $sql = $db->prepare($query);
    $sql->execute();
    
}

function adjustPhone($phone){
    
    printArray($phone);
    
    if(!isset($phone) || $phone == ''){
        
        return '';
        
    }
    
    $phone = trim(preg_replace('/[^0-9]/', '', $phone));
    
    if(substr($phone, 0,1) != '+'){
       $phone = '+'.$phone;
    }
    
    printArray($phone);
    
    $phone = str_replace("+55", "", $phone);
    $phone = str_replace("+", "", $phone);
    
    printArray($phone);
    
    if(substr($phone, 0,1) != '0'){
        $phone = '+55'.$phone;
    }else{
        $phone = '+55'.substr($phone, 1);
    }
    
    printArray($phone);
    

    if(strlen($phone) == 14 ){
        return $phone;
        
    }elseif(strlen($phone) == 13 ){
        
        if(substr($phone, 0,1) == '+'){
            
            if(substr($phone, 5,1) == '7' || substr($phone, 5,1) == '8' || substr($phone, 5,1) == '9'){
                
                return substr($phone, 0,5).'9'.substr($phone, 5);
                
            }else{
            
                return $phone;
                
            }
            
        }else{
            return '+'.$phone;
        }
        
    }elseif(strlen($phone) < 13 ){
        
        if(substr($phone, 0,3) != '+55'){
            
            $phone = str_replace("+", "", $phone);
            
            if(strlen($phone) == 11){
                
                return '+55'.$phone;
                
            }elseif(strlen($phone) < 11){
                if(substr($phone, 2,1) == '7' || substr($phone, 2,1) == '8' || substr($phone, 2,1) == '9'){
                    return '+55'.substr($phone, 0,2).'9'.substr($phone, 2);
                }else{
                    return '+55'.$phone;
                }
            }else{
            
                return $phone;
            
            }
        }else{
            
            return $phone;
            
        }
        
        
        
    }else{    
        return $phone;
    }
    
}

function validateEmail($email) {
    
    $email = explode(";", $email)[0];
    
    if(!filter_var($email, FILTER_VALIDATE_EMAIL) && trim($email) != '') {

        echo "E-mail ".$email." inválido";
        writeToLog("E-mail ".$email." inválido", "ERRO ".date('Y-m-d H:i:s'));

        return "";
    }else{
        return $email;
    }
}

function phonesUpdate($oldPhones, $newPhone, $type = 'WORK', $phonEmail = 'PHONE'){
    
    printArray($oldPhones);
    
    if(isset($newPhone) && $newPhone != ''){
        
        if($phonEmail == 'PHONE'){
    
            $found = false;
            foreach($oldPhones as $key => $value){
                if(adjustPhone($value['VALUE']) == adjustPhone($newPhone)){
                    $found = true; 
                    
                    $oldPhones[$key]['VALUE'] = adjustPhone($value['VALUE']);
                }else{
                    
                    $oldPhones[$key]['VALUE'] = adjustPhone($value['VALUE']);
                }
            }
            
            if($found == false){
                
                $oldPhones[] = array(
                                'VALUE_TYPE' => $type,
                                'VALUE'      => adjustPhone($newPhone)
                    );
                
            }
            
            return $oldPhones;
            
        }elseif($phonEmail == 'EMAIL'){
            
            $found = false;
            foreach($oldPhones as $key => $value){
                if($value['VALUE'] == validateEmail($newPhone)){
                    $found = true;        
                }
            }
            
            if($found == false){
                
                $oldPhones[] = array(
                                'VALUE_TYPE' => $type,
                                'VALUE'      => validateEmail($newPhone)
                    );
                
            }
            
            return $oldPhones;
            
        }else{
            
            $found = false;
            foreach($oldPhones as $key => $value){
                if($value['VALUE'] == $newPhone){
                    $found = true;        
                }
            }
            
            if($found == false){
                
                $oldPhones[] = array(
                                'VALUE_TYPE' => $type,
                                'VALUE'      => $newPhone
                    );
                
            }
            
            return $oldPhones;
            
        }
        
    }else{
        return $oldPhones;
    }
    
}

function getUltima($dta){
    
    // substr(string $string, int $offset, ?int $length = null)  AAAAMMDD
    
    return substr($dta, 0, 4)."-".substr($dta, 4, 2)."-".substr($dta, 6, 2);
    
}


function formatar_cpf_cnpj($doc) {

    $doc = preg_replace("/[^0-9]/", "", $doc);
    $qtd = strlen($doc);

    if ($qtd >= 11) {

        if ($qtd === 11) {

            $docFormatado = substr($doc, 0, 3).
            '.'.
            substr($doc, 3, 3).
            '.'.
            substr($doc, 6, 3).
            '.'.
            substr($doc, 9, 2);
        } else {
            $docFormatado = substr($doc, 0, 2).
            '.'.
            substr($doc, 2, 3).
            '.'.
            substr($doc, 5, 3).
            '/'.
            substr($doc, 8, 4).
            '-'.
            substr($doc, -2);
        }

        return $docFormatado;

    } else {
        return 'Documento invalido';
    }
}


function getName($nome, $parte) {
    
        $nome = str_replace('(', '', str_replace(')', '', $nome));

	    $parts = array();

	    while ( strlen( trim($nome)) > 0 ) {
	        $nome = trim($nome);
	        $string = preg_replace('#.*\s([\w-]*)$#', '$1', $nome);
	        $parts[] = $string;
	        $nome = trim( preg_replace('#'.$string.'#', '', $nome ) );
	    }

	    if (empty($parts)) {
	        return false;
	    }

	    $parts = array_reverse($parts);
	    $nome = array();
	    $nome['first_nomealuno'] = $parts[0];

	    $nome['middle_nomealuno'] = (isset($parts[2])) ? $parts[1] : '';
	    $nome['last_nomealuno'] = (isset($parts[2])) ? $parts[2] : ( isset($parts[1]) ? $parts[1] : '');

	    if($parte == 1){

	        return reset($parts);

	    }elseif($parte == 2){
	        $middle = '';

	        if(count($parts)>2){

	            for ($i=1; $i <= count($parts)-2; $i++) { 
	                $middle = $middle." ".$parts[$i];
	            }
	            return $middle;


	        }else{
	            return '';
	        }

	    }elseif($parte == 3){

	        return end($parts);

	    }else{
	        return '';
	    }

}

function addContact($fields, $print = 0){
    
    $queryUrl = "crm.contact.add";

    $queryArray = array("fields" => $fields);  
                        
    $result = getJson($queryUrl, $queryArray, $print);

	return $result;
	
}

function updateContact($fields, $id, $print = 0){
    
    if(isset($id) && $id >0 ){
        $queryUrl = "crm.contact.update";
    
        $queryArray = array("fields" => $fields, "ID" => $id);  
        
        printArray($queryArray);
        
        $result = getJson($queryUrl, $queryArray, $print);
    
    	return $result;
    }else{
        exit();
    }
	
}

function sendToDrupal($url, $db){
    
    global $obsProcessando;
    global $idProcessando;
    
    $query = "https://live-tese-prime.pantheonsite.io/tese_user/create?".$url;
    
    printArray($query);
    
    $curl = curl_init();
    
    curl_setopt_array($curl, [
      CURLOPT_URL => $query,
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_ENCODING => "",
      CURLOPT_MAXREDIRS => 10,
      CURLOPT_TIMEOUT => 30,
      CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
      CURLOPT_CUSTOMREQUEST => "POST",
      CURLOPT_POSTFIELDS => "",
    ]);
    
    $response = curl_exec($curl);
    $err = curl_error($curl);
    
    curl_close($curl);
    
    if ($err) {
      echo "cURL Error #:" . $err;
      updateStatus($idProcessando, $db, 3, $obsProcessando, $err);
    } else {
      return $response;
      updateStatus($idProcessando, $db, 2, $obsProcessando, $err);
    }

}



/***********************************************************************/

if($_REQUEST['tkn'] == '4rbJsuDdw2X0GX8'){
    
    $dados = getRequestBd($db);

    foreach($dados as $key => $value){
        
        printArray($value);
        
        $idProcessando = $value['id'];
        $obsProcessando = $value['obs'];
        
        if(checkStatus($value['id'], $db, 0)){
            updateStatus($value['id'], $db, 1, $value['obs']);
        }else{
            continue;
        }

        if($request == $value['data']){
            updateStatus($value['id'], $db, 3, $value['obs'], "REPETIDO");
            continue;
        }else{
            $request = $value['data'];
        }
        
        $data = unserialize($value['data']);
        
        printArray($data);
        
        $url = "email={$data['email']}&name=".urlencode($data['name'])."&cpf={$data['cpf']}&psspte={$data['psspte']}";
        
        $retorno = sendToDrupal($url, $db);
        
        $arRetorno = json_decode($retorno, 1);
        
        if($arRetorno['STATUS'] == 'ERRO'){
            $mensagem = '[B]ERRO SISTEMA DE EXAMES: (TESTES)[/B]
'.$arRetorno['ERROR_DESC'];

            addFeed(explode("_",$data['document_id'][2])[1], $mensagem, '3');
            
            $fields = array("UF_CRM_1694871764480" => $arRetorno['VALUE']); //ID TESTE - Exames (BYTEBIO)
            updateContact($fields, explode("_",$data['document_id'][2])[1], 1);
            
        }elseif($arRetorno['STATUS'] == 'SUCESSO'){
        
            // $fields = array("UF_CRM_1597844527467" => $arRetorno['VALUE']); //ID Externo - Exames  - CAMPO OFICIAL
            $fields = array("UF_CRM_1694871764480" => $arRetorno['VALUE']); //ID TESTE - Exames (BYTEBIO)
            
            updateContact($fields, explode("_",$data['document_id'][2])[1], 1);
            
            $mensagem = '[B]USUARIO CRIADO SISTEMA DE EXAMES: (TESTES)[/B]
UID -> '.$arRetorno['VALUE'];

            addFeed(explode("_",$data['document_id'][2])[1], $mensagem, '3');
        
        }
        
        updateStatus($value['id'], $db, 2, $value['obs']);

    }

}

        









